Package: corral
Title: Correspondence Analysis for Single Cell Data
Version: 1.23.0
Date: 2023-02-09
Authors@R: 
    c(person(given = "Lauren", family = "Hsu",
             role = c("aut", "cre"),
             email = "lrnshoe@gmail.com",
             comment = c(ORCID = "0000-0002-6035-7381")),
     person(given = "Aedin", family = "Culhane",
             role = c("aut"),
             email = "Aedin.Culhane@ul.ie",
             comment = c(ORCID = "0000-0002-1395-9734")))
Description: 
    Correspondence analysis (CA) is a matrix factorization method, and is 
    similar to principal components analysis (PCA). Whereas PCA is designed for 
    application to continuous, approximately normally distributed data, CA is 
    appropriate for non-negative, count-based data that are in the same additive scale. 
    The corral package implements CA for dimensionality reduction of a single matrix of 
    single-cell data, as well as a multi-table adaptation of CA that leverages 
    data-optimized scaling to align data generated from different sequencing platforms 
    by projecting into a shared latent space. corral utilizes sparse matrices and a 
    fast implementation of SVD, and can be called directly on Bioconductor objects 
    (e.g., SingleCellExperiment) for easy pipeline integration.
    The package also includes additional options, including variations of CA to
    address overdispersion in count data (e.g., Freeman-Tukey chi-squared residual), 
    as well as the option to apply CA-style processing to continuous data 
    (e.g., proteomic TOF intensities) with the Hellinger distance adaptation of CA.
Imports: ggplot2, ggthemes, grDevices, gridExtra, irlba, Matrix,
        methods, MultiAssayExperiment, pals, reshape2,
        SingleCellExperiment, SummarizedExperiment, transport
Suggests: ade4, BiocStyle, CellBench, DuoClustering2018, knitr,
        rmarkdown, scater, testthat
License: GPL-2
RoxygenNote: 7.1.2
VignetteBuilder: knitr
biocViews: BatchEffect, DimensionReduction, GeneExpression,
        Preprocessing, PrincipalComponent, Sequencing, SingleCell,
        Software, Visualization
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/corral
git_branch: devel
git_last_commit: 757a1c2
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:12:05 UTC; biocbuild
Author: Lauren Hsu [aut, cre] (ORCID: <https://orcid.org/0000-0002-6035-7381>),
  Aedin Culhane [aut] (ORCID: <https://orcid.org/0000-0002-1395-9734>)
Maintainer: Lauren Hsu <lrnshoe@gmail.com>
